<?php

/**
 * @file
 * Administrative functions for user types.
 */

/**
 * Settings page callback.
 */
function user_types_settings() {
  $form['user_registration'] = array(
    '#type' => 'fieldset',
    '#title' => t('User registration page'),
  );
  $form['user_registration']['user_types_hijack_user_registration'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hijack user registration page'),
    '#default_value' => variable_get('user_types_hijack_user_registration', 0),
    '#description' => t('Toggle the checkbox to display a screen with all user types and their description when going to user/registration. A user then chooses which type he wants to be.'),
  );
  $form['user_registration']['user_types_untyped_show_descr'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show description'),
    '#description' => t('If you don\'t hijack the user/register page, show description at top of the form.'),
    '#default_value' => variable_get('user_types_untyped_show_descr', 0),
  );

  $form['untyped'] = array(
    '#type' => 'fieldset',
    '#title' => t('Untyped user'),
  );
  $form['untyped']['user_types_untyped_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name for untyped users'),
    '#default_value' => variable_get('user_types_untyped_name', t('Normal user')),
    '#required' => TRUE,
  );
  $form['untyped']['user_types_untyped_description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description for untyped users'),
    '#default_value' => variable_get('user_types_untyped_description', ''),
  );
  $form['untyped']['user_types_untyped_auto_role'] = array(
    '#type' => 'select',
    '#title' => 'Auto assign role',
    '#options' => user_types_get_available_roles(),
    '#default_value' => variable_get('user_types_untyped_auto_role', 0),
  );

  $form['welcome_mail'] = array(
    '#type' => 'fieldset',
    '#title' => t('Welcome mail'),
    '#description' => t('The welcome mail will be altered when a user registers on the site, not when an administrator registers a new user.<br/>Corresponding mail keys are: user-register-welcome and user-register-approval-user.'),
  );
  $form['welcome_mail']['user_types_untyped_mail_check'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use welcome mail.'),
    '#description' => t('Toggle this checkbox to use the text underneath as the welcome mail for the untyped user.'),
    '#default_value' => variable_get('user_types_untyped_mail_check', 0),
  );
  $form['welcome_mail']['user_types_untyped_mail_text'] = array(
    '#type' => 'textarea',
    '#title' => t('Welcome mail on registration'),
    '#rows' => 15,
    '#description' => t('Available variables are:') .' !username, !site, !password, !uri, !uri_brief, !mailto, !login_uri, !edit_uri, !login_url.',
    '#default_value' => variable_get('user_types_untyped_mail_text', _user_mail_text('register_no_approval_required_body')),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings'),
  );
  return $form;
}

/**
 * Settings page submit callback.
 */
function user_types_settings_submit($form, &$form_state) {
  variable_set('user_types_untyped_name', $form_state['values']['user_types_untyped_name']);
  variable_set('user_types_untyped_description', $form_state['values']['user_types_untyped_description']);
  variable_set('user_types_hijack_user_registration', $form_state['values']['user_types_hijack_user_registration']);
  variable_set('user_types_untyped_auto_role', $form_state['values']['user_types_untyped_auto_role']);
  variable_set('user_types_untyped_show_descr', $form_state['values']['user_types_untyped_show_descr']);
  variable_set('user_types_untyped_mail_check', $form_state['values']['user_types_untyped_mail_check']);
  variable_set('user_types_untyped_mail_text', $form_state['values']['user_types_untyped_mail_text']);
  menu_rebuild();
  drupal_set_message('Saved user types settings.');
}

/**
 * @return list of user types.
 */
function user_types_admin_page() {
  $header = array(
    array('data' => t('Name')),
    array('data' => t('Description')),
    array('data' => t('Enabled')),
    array('data' => t('Show description')),
    array('data' => t('Auto role')),
    array('data' => t('Operations')),
  );

  $rows = array();

  $user_types = user_types_list(FALSE, TRUE);
  $roles = user_types_get_available_roles();
  foreach ($user_types as $user_type) {
    $enabled = ($user_type['enabled'] == 1) ? t('Yes') : t('No');
    $show_de = ($user_type['show_descr'] == 1) ? t('Yes') : t('No');
    $rows[] = array(
      $user_type['name'],
      $user_type['description'],
      $enabled,
      $show_de,
      $roles[$user_type['role']],
      l('Edit', 'admin/user/user_types/form/'. $user_type['id']) .' - '.
      l('List users', 'admin/user/user_types/list_users', array('query' => 'user_type='. $user_type['id'])),
    );
  }
  $show_descr = variable_get('user_types_untyped_show_descr', 0);
  $show_descr = ($show_descr == 1) ? t('Yes') : t('No');
  $rows[] = array(
    variable_get('user_types_untyped_name', t('Normal user')),
    variable_get('user_types_untyped_description', ''),
    t('Yes'),
    $show_descr,
    $roles[variable_get('user_types_untyped_auto_role', 0)],
    l('Edit', 'admin/user/user_types/settings') .' - '.
    l('List users', 'admin/user/user_types/list_users', array('query' => 'user_type=-1')),
  );

  $output = t('<a href="!url">Add a user type</a>', array('!url' => url('admin/user/user_types/form')));
  $output .= theme('table', $header, $rows);
  return $output;
}

/**
 * Add/Edit user type form.
 */
function user_types_type_form($form_state, $user_type_id = NULL) {
  $user_type = db_fetch_object(db_query('SELECT user_type_id, name, description, enable_registration, role, show_descr FROM {user_types_type} WHERE user_type_id = %d', $user_type_id));
  if (!empty($user_type->user_type_id)) {
    $form['user_type_id'] = array(
      '#type' => 'value',
      '#value' => $user_type->user_type_id,
    );
    drupal_set_title(t('Edit user type: @type', array('@type' => $user_type->name)));
    $check = variable_get('user_types_mail_check_'. $user_type->user_type_id, 0);
    $mail = variable_get('user_types_mail_text_'. $user_type->user_type_id, _user_mail_text('register_no_approval_required_body'));
  }
  else {
    $check = 0;
    $mail = _user_mail_text('register_no_approval_required_body');
    $user_type->name = '';
    $user_type->description = '';
    $user_type->enable_registration = 0;
    $user_type->role = 0;
    $user_type->show_descr = 0;
    $form['user_type_form_action'] = array(
      '#type' => 'value',
      '#value' => 'add',
    );
    drupal_set_title('Add user type');
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#required' => TRUE,
    '#default_value' => $user_type->name,
  );
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('Enter a description if you choose to display all user types on the registration page.'),
    '#default_value' => $user_type->description,
  );
  $form['enable_registration'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable registration'),
    '#default_value' => $user_type->enable_registration,
  );
  $form['show_descr'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show description'),
    '#description' => t('If you don\'t hijack the user/register page, show description at top of the form.'),
    '#default_value' => $user_type->show_descr,
  );
  $form['role'] = array(
    '#type' => 'select',
    '#options' => user_types_get_available_roles(),
    '#title' => t('Auto assign role'),
    '#default_value' => $user_type->role,
  );
  $form['welcome_mail'] = array(
    '#type' => 'fieldset',
    '#title' => t('Welcome mail'),
    '#description' => t('The welcome mail will be altered when a user registers on the site, not when an administrator registers a new user.<br/>Corresponding mail keys are: user-register-welcome and user-register-approval-user.'),
  );
  $form['welcome_mail']['user_types_mail_check'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use welcome mail.'),
    '#description' => t('Toggle this checkbox to use the text underneath as the welcome mail for this user type.'),
    '#default_value' => $check,
  );
  $form['welcome_mail']['user_types_mail_text'] = array(
    '#type' => 'textarea',
    '#title' => t('Welcome mail on registration'),
    '#rows' => 15,
    '#description' => t('Available variables are:') .' !username, !site, !password, !uri, !uri_brief, !mailto, !login_uri, !edit_uri, !login_url.',
    '#default_value' => $mail,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save user type'),
    '#submit' => array('user_types_type_form_submit'),
  );
  if (!empty($user_type->user_type_id)) {
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Remove user type'),
      '#submit' => array('user_types_type_delete_submit'),
    );
  }
  return $form;
}

/**
 * Save user type form submit.
 */
function user_types_type_form_submit($form, &$form_state) {

  $user_type->name = $form_state['values']['name'];
  $user_type->description = $form_state['values']['description'];
  $user_type->enable_registration = $form_state['values']['enable_registration'];
  $user_type->show_descr = $form_state['values']['show_descr'];
  $user_type->role = $form_state['values']['role'];
  if (!isset($form_state['values']['user_type_id'])) {
    drupal_write_record('user_types_type', $user_type);
    $user_type_id = db_last_insert_id('user_types_type', 'user_type_id');

    // Add all existing profile fields to profile_fields table and set enabled to 0
    $result = db_query("SELECT fid FROM {profile_fields}");
    while ($row = db_fetch_object($result)) {
      db_query("INSERT INTO {user_types_profile_fields} VALUES (%d, %d, 0, 0)", $user_type_id, $row->fid);
    }
  }
  else {
    $user_type->user_type_id = $form_state['values']['user_type_id'];
    $user_type_id = $user_type->user_type_id;
    drupal_write_record('user_types_type', $user_type, 'user_type_id');
  }
  // Save mail
  variable_set('user_types_mail_check_'. $user_type_id, $form_state['values']['user_types_mail_check']);
  variable_set('user_types_mail_text_'. $user_type_id, $form_state['values']['user_types_mail_text']);
  menu_rebuild();
  drupal_set_message(t('Saved %name user type.', array('%name' => $form_state['values']['name'])));
  $form_state['redirect'] = array('admin/user/user_types');
}

/**
 * Delete submit callback
 */
function user_types_type_delete_submit($form, &$form_state) {
  $form_state['redirect'] = array('admin/user/user_types/delete/'. $form_state['values']['user_type_id']);
}

/**
 * Delete user type form.
 */
function user_types_delete_form($form_state, $user_type_id) {
  $user_type = db_fetch_object(db_query('SELECT user_type_id, name FROM {user_types_type} WHERE user_type_id = %d', $user_type_id));
  if (empty($user_type)) {
    drupal_set_message(t('Invalid user type.'), 'error');
    return array();
  }

  $form['user_type_id'] = array(
    '#type' => 'value',
    '#value' => $user_type->user_type_id,
  );
  $form['name'] = array(
    '#type' => 'value',
    '#value' => $user_type->name,
  );

  return confirm_form($form, t('Are you sure you want to delete the %name user type?', array('%name' => $user_type->name)), 'admin/user/user_types/form/'. $user_type->user_type_id);
}

/**
 * Delete user type submit form.
 */
function user_types_delete_form_submit($form, &$form_state) {
  db_query("DELETE FROM {user_types_type} WHERE user_type_id = %d", $form_state['values']['user_type_id']);
  db_query("DELETE FROM {user_types_profile_fields} WHERE user_type_id = %d", $form_state['values']['user_type_id']);
  db_query("UPDATE {user_types_user} set user_type_id = '-1' WHERE user_type_id = %d", $form_state['values']['user_type_id']);
  variable_del('user_types_mail_check_'. $form_state['values']['user_type_id']);
  variable_del('user_types_mail_text_'. $form_state['values']['user_type_id']);
  menu_rebuild();
  drupal_set_message(t('Deleted %name user type.', array('%name' => $form_state['values']['name'])));
  $form_state['redirect'] = array('admin/user/user_types');
}

/**
 * Menu callback to list users overview, operations and filter form.
 */
function user_types_list_users() {
  $output = drupal_get_form('user_types_list_filter_form');
  $output .= drupal_get_form('user_types_list_users_form');
  return $output;
}

/**
 * Filter form.
 */
function user_types_list_filter_form() {
  if (!isset($_SESSION['user_types_filter'])) $_SESSION['user_types_filter'] = 'all_user_types';
  if (isset($_GET['user_type'])) $_SESSION['user_types_filter'] = $_GET['user_type'];

  $options['System']['all_user_types'] = t('All users');
  $options['System'][-1] = variable_get('user_types_untyped_name', t('Normal user'));
  $options['User types'] = user_types_list();

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter on user types'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['filters']['user_types'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#default_value' => $_SESSION['user_types_filter'],
  );
  $form['filters']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#submit' => array('user_types_list_filter_form_submit'),
  );
  $form['filters']['download'] = array(
    '#type' => 'submit',
    '#value' => t('Download csv'),
    '#submit' => array('user_types_list_filter_csv_submit'),
  );
  return $form;
}

/**
 * Filter form submit.
 */
function user_types_list_filter_form_submit($form, &$form_state) {
  $_SESSION['user_types_filter'] = $form_state['values']['user_types'];
}

/**
 * Filter form cvs submit
 */
function user_types_list_filter_csv_submit($form, &$form_state) {
  $_SESSION['user_types_filter'] = $form_state['values']['user_types'];
  // Get user types and mysql result
  $result = user_types_get_sql_resource($form_state['values']['user_types']);

  $csv = user_types_to_csv(array('Uid', 'Username', 'User type'));
  while ($row = db_fetch_object($result['resource'])) {
    $item = array(
      $row->uid,
      $row->name,
      $result['user_types_all'][$row->user_type_id],
    );
    $csv .= user_types_to_csv($item);
  }
  ini_set('zlib.output_compression', 'Off');
  $filename = is_numeric($form_state['values']['user_types']) ? $result['user_types_all'][$form_state['user_types']] : 'all_users';
  drupal_set_header('Content-Disposition: attachment; filename="'. basename($filename) .'.csv"');
  drupal_set_header('Pragma: public');
  drupal_set_header("Expires: Mon, 26 Jul 1997 05:00:00 GMT"); // Date in the past
  drupal_set_header('Last-Modified: '. gmdate('D, d M Y H:i:s') .' GMT');
  drupal_set_header('Cache-Control: no-store, no-cache, must-revalidate'); // HTTP/1.1
  drupal_set_header('Cache-Control: pre-check=0, post-check=0, max-age=0'); // HTTP/1.1
  drupal_set_header("Pragma: no-cache");
  drupal_set_header("Expires: 0");
  drupal_set_header('Content-Transfer-Encoding: none');
  drupal_set_header('Content-Type: application/vnd.ms-excel;'); // This should work for IE & Opera
  drupal_set_header("Content-type: application/x-msexcel"); // This should work for the rest
  print $csv;
  exit();
}

/**
 * Display list with users.
 */
function user_types_list_users_form() {
  if (!isset($_SESSION['user_types_filter'])) $_SESSION['user_types_filter'] = 'all_user_types';
  $user_type = $_SESSION['user_types_filter'];
  if (isset($_GET['user_type'])) $user_type = $_GET['user_type'];

  $header = array(
    array(),
    array('data' => t('Username'), 'field' => 'u.name'),
    array('data' => t('User type')),
    array('data' => t('Status'), 'field' => 'u.status'),
    array('data' => t('Member for'), 'field' => 'u.created', 'sort' => 'desc'),
  );

  // Get user types and mysql result
  $result = user_types_get_sql_resource($user_type, $header);
  $user_types_all = $result['user_types_all'];

  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Update options'),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $options = array();

  $add_types = array();
  foreach ($user_types_all as $key => $value) {
    $add_types[$key] = $value;
  }

  if (count($user_types_all)) {
    $role_operations = array(
      t('Change type of the selected users') => $add_types,
    );
    $options += $role_operations;
  }

  $form['options']['operation'] = array(
    '#type' => 'select',
    '#options' => $options,
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
  );
  $form['options']['description'] = array(
    '#type' => 'item',
    '#value' => t('Changing the user type will <strong>lose data</strong> in fields which are removed.'),
  );

  $destination = drupal_get_destination();

  $status = array(t('blocked'), t('active'));

  while ($account = db_fetch_object($result['resource'])) {
    $accounts[$account->uid] = '';
    $form['name'][$account->uid] = array('#value' => theme('username', $account));
    $form['user_type'][$account->uid] = array('#value' => $user_types_all[$account->user_type_id]);
    $form['status'][$account->uid] =  array('#value' => $status[$account->status]);
    $form['member_for'][$account->uid] = array('#value' => format_interval(time() - $account->created));
  }
  $form['accounts'] = array(
    '#type' => 'checkboxes',
    '#options' => $accounts
  );

  $form['pager'] = array('#value' => theme('pager', NULL, 50, 0));

  return $form;
}

/**
 * Theme user types list overview.
 */
function theme_user_types_list_users_form($form) {
  $header = array(
    theme('table_select_header_cell'),
    array('data' => t('Username'), 'field' => 'u.name'),
    array('data' => t('User type')),
    array('data' => t('Status'), 'field' => 'u.status'),
    array('data' => t('Member for'), 'field' => 'u.created', 'sort' => 'desc'),
  );

  $output = drupal_render($form['options']);
  if (isset($form['name']) && is_array($form['name'])) {
    foreach (element_children($form['name']) as $key) {
      $rows[] = array(
        drupal_render($form['accounts'][$key]),
        drupal_render($form['name'][$key]),
        drupal_render($form['user_type'][$key]),
        drupal_render($form['status'][$key]),
        drupal_render($form['member_for'][$key]),
      );
    }
  }
  else  {
    $rows[] = array(array('data' => t('No users available.'), 'colspan' => '7'));
  }

  $output .= theme('table', $header, $rows);
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }

  $output .= drupal_render($form);

  return $output;
}

/**
 * User list operations submit.
 */
function user_types_list_users_form_submit($form, &$form_state) {
  $accounts = array_filter($form_state['values']['accounts']);
  $user_type_id = $form_state['values']['operation'];
  foreach ($accounts as $key => $value) {
    user_types_change_type($key, $user_type_id, FALSE);
  }
  drupal_set_message(t('The update has been performed.'));
}

/**
 * Helper function to convert an array with items to a csv line.
 */
function user_types_to_csv($array) {
  $output = '"'. user_types_quote_csv(array_shift($array)) .'"';
  foreach ($array as $item) {
    $output .= ',"'. user_types_quote_csv($item) .'"';
  }
  $output .= "\n";

  return $output;
}

/**
 * Helper function to strip newlines.
 */
function user_types_quote_csv($string) {
  $string = str_replace('"', '""', strip_tags($string));
  $string = str_replace("\r\n", "\n", $string);
  $string = str_replace("\r", "\n", $string);

  return $string;
}

/**
 * Helper function to get mysql resource with list of users
 */
function user_types_get_sql_resource($user_type, $header = NULL) {
  $sql = "SELECT u.name, u.uid, u.status, u.access, u.created, ut.user_type_id FROM {users} u LEFT JOIN {user_types_user} ut on ut.uid = u.uid WHERE u.uid <> 0";
  $args = array();
  if (is_numeric($user_type)) {
    $sql .= ' AND ut.user_type_id = %d';
    $args[] = $user_type;
  }
  if ($header)
  $sql .= tablesort_sql($header);

  $user_types = user_types_list();
  $user_types_all[-1] = variable_get('user_types_untyped_name', t('Normal user'));
  $user_types_all += $user_types;

  if ($header)
  $result = pager_query($sql, 50, 0, NULL, $args);
  else
  $result = db_query($sql, $args);

  return array('resource' => $result, 'user_types_all' => $user_types_all);
}

/**
 * Helper function to get available roles.
 */
function user_types_get_available_roles() {
  $roles[0] = t('No role');
  $roles += user_roles();
  unset($roles[1]);
  unset($roles[2]);
  return $roles;
}
